@@ -48,6 +48,7 @@
 #include <juce_graphics/native/juce_CoreGraphicsHelpers_mac.h>
 #include <juce_audio_basics/native/juce_CoreAudioLayouts_mac.h>
 #include <juce_audio_basics/native/juce_CoreAudioTimeConversions_mac.h>
+#include <juce_audio_basics/native/juce_AudioWorkgroup_mac.h>
 #include <juce_audio_processors/format_types/juce_LegacyAudioParameter.cpp>
 #include <juce_audio_processors/format_types/juce_AU_Shared.h>
 
@@ -67,7 +68,7 @@ JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wnullability-completeness")
 
 using namespace juce;
 
-struct AudioProcessorHolder  : public ReferenceCountedObject
+struct AudioProcessorHolder final : public ReferenceCountedObject
 {
     AudioProcessorHolder() = default;
     explicit AudioProcessorHolder (std::unique_ptr<AudioProcessor> p) : processor (std::move (p)) {}
@@ -94,12 +95,17 @@ private:
     AudioProcessorHolder& operator= (AudioProcessorHolder&) = delete;
 };
 
+#if ! JUCE_AUDIOWORKGROUP_TYPES_AVAILABLE
+ struct AudioUnitRenderContext;
+ typedef void (^AURenderContextObserver) (const AudioUnitRenderContext*);
+#endif
+
 //==============================================================================
 //=========================== The actual AudioUnit =============================
 //==============================================================================
-class JuceAudioUnitv3  : public AudioProcessorListener,
-                         public AudioPlayHead,
-                         private AudioProcessorParameter::Listener
+class JuceAudioUnitv3 final : public AudioProcessorListener,
+                              public AudioPlayHead,
+                              private AudioProcessorParameter::Listener
 {
 public:
     JuceAudioUnitv3 (const AudioProcessorHolder::Ptr& processor,
@@ -187,6 +193,15 @@ public:
 
         internalRenderBlock = CreateObjCBlock (this, &JuceAudioUnitv3::renderCallback);
 
+       #if JUCE_AUDIOWORKGROUP_TYPES_AVAILABLE
+        renderContextObserver = ^(const AudioUnitRenderContext* context)
+        {
+            getAudioProcessor().audioWorkgroupContextChanged (makeRealAudioWorkgroup (context->workgroup));
+        };
+       #else
+        renderContextObserver = ^(const AudioUnitRenderContext*) {};
+       #endif
+
         processor.setRateAndBufferSizeDetails (kDefaultSampleRate, static_cast<int> (maxFrames));
         processor.prepareToPlay (kDefaultSampleRate, static_cast<int> (maxFrames));
         processor.addListener (this);
@@ -312,9 +327,9 @@ public:
     }
 
     //==============================================================================
-    AUAudioUnitBusArray* getInputBusses()             const { return inputBusses.get();  }
-    AUAudioUnitBusArray* getOutputBusses()            const { return outputBusses.get(); }
-    NSArray<NSNumber*>* getChannelCapabilities()      const { return channelCapabilities.get(); }
+    AUAudioUnitBusArray* getInputBusses()                const { return inputBusses.get();  }
+    AUAudioUnitBusArray* getOutputBusses()               const { return outputBusses.get(); }
+    NSArray<NSNumber*>* getChannelCapabilities()         const { return channelCapabilities.get(); }
 
     bool shouldChangeToFormat (AVAudioFormat* format, AUAudioUnitBus* auBus)
     {
@@ -383,8 +398,10 @@ public:
     }
 
     //==============================================================================
-    AUInternalRenderBlock getInternalRenderBlock() const  { return internalRenderBlock; }
-    bool getRenderingOffline()                     const  { return getAudioProcessor().isNonRealtime(); }
+    AUInternalRenderBlock   getInternalRenderBlock()      const { return internalRenderBlock; }
+    AURenderContextObserver getInternalContextObserver()  const { return renderContextObserver; }
+
+    bool getRenderingOffline()                            const { return getAudioProcessor().isNonRealtime(); }
     void setRenderingOffline (bool offline)
     {
         auto& processor = getAudioProcessor();
@@ -724,7 +741,7 @@ public:
     AUAudioUnit* getAudioUnit() const { return au; }
 
 private:
-    struct Class  : public ObjCClass<AUAudioUnit>
+    struct Class final : public ObjCClass<AUAudioUnit>
     {
         Class() : ObjCClass<AUAudioUnit> ("AUAudioUnit_")
         {
@@ -741,7 +758,7 @@ private:
                 AUAudioUnit* self = _self;
 
                 self = ObjCMsgSendSuper<AUAudioUnit, AUAudioUnit*, AudioComponentDescription,
-                                        AudioComponentInstantiationOptions, NSError**> (self, @selector(initWithComponentDescription:options:error:), descr, options, error);
+                                        AudioComponentInstantiationOptions, NSError**> (self, @selector (initWithComponentDescription:options:error:), descr, options, error);
 
                 setThis (self, juceAU);
                 return self;
@@ -772,7 +789,7 @@ private:
                 {
                     WaitableEvent deletionEvent;
 
-                    struct AUDeleter  : public CallbackMessage
+                    struct AUDeleter final : public CallbackMessage
                     {
                         AUDeleter (id selfToDelete, WaitableEvent& event)
                             : parentSelf (selfToDelete), parentDeletionEvent (event)
@@ -842,7 +859,11 @@ private:
 
             //==============================================================================
             addMethod (@selector (contextName),                             [] (id self, SEL)                                                   { return _this (self)->getContextName(); });
-            addMethod (@selector (setContextName:),                         [](id self, SEL, NSString* str)                                     { return _this (self)->setContextName (str); });
+            addMethod (@selector (setContextName:),                         [] (id self, SEL, NSString* str)                                    { return _this (self)->setContextName (str); });
+
+           #if JUCE_AUDIOWORKGROUP_TYPES_AVAILABLE
+            addMethod (@selector (renderContextObserver),                   [] (id self, SEL)                                                   { return _this (self)->getInternalContextObserver(); });
+           #endif
 
             //==============================================================================
             if (@available (macOS 10.13, iOS 11.0, *))
@@ -1162,16 +1183,16 @@ private:
         @try
         {
             // Create methods in AUParameterTree return unretained objects (!) -> see Apple header AUAudioUnitImplementation.h
-            param.reset([[AUParameterTree createParameterWithIdentifier: juceStringToNS (getParameterIdentifier())
-                                                                   name: juceStringToNS (name)
-                                                                address: address
-                                                                    min: 0.0f
-                                                                    max: getMaximumParameterValue (parameter)
-                                                                   unit: unit
-                                                               unitName: nullptr
-                                                                  flags: flags
-                                                           valueStrings: valueStrings.get()
-                                                    dependentParameters: nullptr]
+            param.reset ([[AUParameterTree createParameterWithIdentifier: juceStringToNS (getParameterIdentifier())
+                                                                    name: juceStringToNS (name)
+                                                                 address: address
+                                                                     min: 0.0f
+                                                                     max: getMaximumParameterValue (parameter)
+                                                                    unit: unit
+                                                                unitName: nullptr
+                                                                   flags: flags
+                                                            valueStrings: valueStrings.get()
+                                                     dependentParameters: nullptr]
                          retain]);
         }
 
@@ -1731,6 +1752,7 @@ private:
     FactoryPresets factoryPresets;
 
     ObjCBlock<AUInternalRenderBlock> internalRenderBlock;
+    ObjCBlock<AURenderContextObserver> renderContextObserver;
 
     AudioUnitHelpers::CoreAudioBufferList audioBuffer;
     AudioUnitHelpers::ChannelRemapper mapper;
