@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-7-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -27,6 +26,14 @@
 namespace juce
 {
 
+static double getStepSize (const Slider& slider)
+{
+    const auto interval = slider.getInterval();
+
+    return ! approximatelyEqual (interval, 0.0) ? interval
+                                                : slider.getRange().getLength() * 0.01;
+}
+
 class Slider::Pimpl   : public AsyncUpdater, // this needs to be public otherwise it will cause an
                                              // error when JUCE_DLL_BUILD=1
                         private Value::Listener
@@ -115,20 +122,34 @@ public:
         return 0.0f;
     }
 
+    void setNumDecimalPlacesToDisplay (int decimalPlacesToDisplay)
+    {
+        fixedNumDecimalPlaces = jmax (0, decimalPlacesToDisplay);
+        numDecimalPlaces = fixedNumDecimalPlaces;
+    }
+
+    int getNumDecimalPlacesToDisplay() const
+    {
+        return fixedNumDecimalPlaces == -1 ? numDecimalPlaces : fixedNumDecimalPlaces;
+    }
+
     void updateRange()
     {
-        // figure out the number of DPs needed to display all values at this
-        // interval setting.
-        numDecimalPlaces = 7;
-
-        if (normRange.interval != 0.0)
+        if (fixedNumDecimalPlaces == -1)
         {
-            int v = std::abs (roundToInt (normRange.interval * 10000000));
+            // figure out the number of DPs needed to display all values at this
+            // interval setting.
+            numDecimalPlaces = 7;
 
-            while ((v % 10) == 0 && numDecimalPlaces > 0)
+            if (! approximatelyEqual (normRange.interval, 0.0))
             {
-                --numDecimalPlaces;
-                v /= 10;
+                int v = std::abs (roundToInt (normRange.interval * 10000000));
+
+                while ((v % 10) == 0 && numDecimalPlaces > 0)
+                {
+                    --numDecimalPlaces;
+                    v /= 10;
+                }
             }
         }
 
@@ -185,21 +206,21 @@ public:
                                newValue);
         }
 
-        if (newValue != lastCurrentValue)
+        if (! approximatelyEqual (newValue, lastCurrentValue))
         {
             if (valueBox != nullptr)
                 valueBox->hideEditor (true);
 
             lastCurrentValue = newValue;
 
-            // (need to do this comparison because the Value will use equalsWithSameType to compare
-            // the new and old values, so will generate unwanted change events if the type changes)
-            if (currentValue != newValue)
+            // Need to do this comparison because the Value will use equalsWithSameType to compare
+            // the new and old values, so will generate unwanted change events if the type changes.
+            // Cast to double before comparing, to prevent comparing as another type (e.g. String).
+            if (! approximatelyEqual (static_cast<double> (currentValue.getValue()), newValue))
                 currentValue = newValue;
 
             updateText();
             owner.repaint();
-            updatePopupDisplay (newValue);
 
             triggerChangeMessage (notification);
         }
@@ -228,12 +249,12 @@ public:
             newValue = jmin (lastCurrentValue, newValue);
         }
 
-        if (lastValueMin != newValue)
+        if (! approximatelyEqual (lastValueMin, newValue))
         {
             lastValueMin = newValue;
             valueMin = newValue;
             owner.repaint();
-            updatePopupDisplay (newValue);
+            updatePopupDisplay();
 
             triggerChangeMessage (notification);
         }
@@ -262,12 +283,12 @@ public:
             newValue = jmax (lastCurrentValue, newValue);
         }
 
-        if (lastValueMax != newValue)
+        if (! approximatelyEqual (lastValueMax, newValue))
         {
             lastValueMax = newValue;
             valueMax = newValue;
             owner.repaint();
-            updatePopupDisplay (valueMax.getValue());
+            updatePopupDisplay();
 
             triggerChangeMessage (notification);
         }
@@ -285,7 +306,7 @@ public:
         newMinValue = constrainedValue (newMinValue);
         newMaxValue = constrainedValue (newMaxValue);
 
-        if (lastValueMax != newMaxValue || lastValueMin != newMinValue)
+        if (! approximatelyEqual (lastValueMax, newMaxValue) || ! approximatelyEqual (lastValueMin, newMinValue))
         {
             lastValueMax = newMaxValue;
             lastValueMin = newMinValue;
@@ -338,8 +359,13 @@ public:
         if (checker.shouldBailOut())
             return;
 
-        if (owner.onValueChange != nullptr)
-            owner.onValueChange();
+        NullCheckedInvocation::invoke (owner.onValueChange);
+
+        if (checker.shouldBailOut())
+            return;
+
+        if (auto* handler = owner.getAccessibilityHandler())
+            handler->notifyAccessibilityEvent (AccessibilityEvent::valueChanged);
     }
 
     void sendDragStart()
@@ -352,8 +378,7 @@ public:
         if (checker.shouldBailOut())
             return;
 
-        if (owner.onDragStart != nullptr)
-            owner.onDragStart();
+        NullCheckedInvocation::invoke (owner.onDragStart);
     }
 
     void sendDragEnd()
@@ -367,20 +392,9 @@ public:
         if (checker.shouldBailOut())
             return;
 
-        if (owner.onDragEnd != nullptr)
-            owner.onDragEnd();
+        NullCheckedInvocation::invoke (owner.onDragEnd);
     }
 
-    struct DragInProgress
-    {
-        DragInProgress (Pimpl& p)  : owner (p)      { owner.sendDragStart(); }
-        ~DragInProgress()                           { owner.sendDragEnd(); }
-
-        Pimpl& owner;
-
-        JUCE_DECLARE_NON_COPYABLE (DragInProgress)
-    };
-
     void incrementOrDecrement (double delta)
     {
         if (style == IncDecButtons)
@@ -393,7 +407,7 @@ public:
             }
             else
             {
-                DragInProgress drag (*this);
+                ScopedDragNotification drag (owner);
                 setValue (newValue, sendNotificationSync);
             }
         }
@@ -407,18 +421,22 @@ public:
                 setValue (currentValue.getValue(), dontSendNotification);
         }
         else if (value.refersToSameSourceAs (valueMin))
+        {
             setMinValue (valueMin.getValue(), dontSendNotification, true);
+        }
         else if (value.refersToSameSourceAs (valueMax))
+        {
             setMaxValue (valueMax.getValue(), dontSendNotification, true);
+        }
     }
 
     void textChanged()
     {
         auto newValue = owner.snapValue (owner.getValueFromText (valueBox->getText()), notDragging);
 
-        if (newValue != static_cast<double> (currentValue.getValue()))
+        if (! approximatelyEqual (newValue, static_cast<double> (currentValue.getValue())))
         {
-            DragInProgress drag (*this);
+            ScopedDragNotification drag (owner);
             setValue (newValue, sendNotificationSync);
         }
 
@@ -434,6 +452,8 @@ public:
             if (newValue != valueBox->getText())
                 valueBox->setText (newValue, dontSendNotification);
         }
+
+        updatePopupDisplay();
     }
 
     double constrainedValue (double value) const
@@ -466,8 +486,10 @@ public:
         if (style != newStyle)
         {
             style = newStyle;
+
             owner.repaint();
             owner.lookAndFeelChanged();
+            owner.invalidateAccessibilityHandler();
         }
     }
 
@@ -589,26 +611,24 @@ public:
             incButton.reset (lf.createSliderButton (owner, true));
             decButton.reset (lf.createSliderButton (owner, false));
 
-            owner.addAndMakeVisible (incButton.get());
-            owner.addAndMakeVisible (decButton.get());
-
-            incButton->onClick = [this] { incrementOrDecrement (normRange.interval); };
-            decButton->onClick = [this] { incrementOrDecrement (-normRange.interval); };
-
-            if (incDecButtonMode != incDecButtonsNotDraggable)
-            {
-                incButton->addMouseListener (&owner, false);
-                decButton->addMouseListener (&owner, false);
-            }
-            else
-            {
-                incButton->setRepeatSpeed (300, 100, 20);
-                decButton->setRepeatSpeed (300, 100, 20);
-            }
-
             auto tooltip = owner.getTooltip();
-            incButton->setTooltip (tooltip);
-            decButton->setTooltip (tooltip);
+
+            auto setupButton = [&] (Button& b, bool isIncrement)
+            {
+                owner.addAndMakeVisible (b);
+                b.onClick = [this, isIncrement] { incrementOrDecrement (isIncrement ? normRange.interval : -normRange.interval); };
+
+                if (incDecButtonMode != incDecButtonsNotDraggable)
+                    b.addMouseListener (&owner, false);
+                else
+                    b.setRepeatSpeed (300, 100, 20);
+
+                b.setTooltip (tooltip);
+                b.setAccessible (false);
+            };
+
+            setupButton (*incButton, true);
+            setupButton (*decButton, false);
         }
         else
         {
@@ -686,8 +706,8 @@ public:
     //==============================================================================
     void handleRotaryDrag (const MouseEvent& e)
     {
-        auto dx = e.position.x - sliderRect.getCentreX();
-        auto dy = e.position.y - sliderRect.getCentreY();
+        auto dx = e.position.x - (float) sliderRect.getCentreX();
+        auto dy = e.position.y - (float) sliderRect.getCentreY();
 
         if (dx * dx + dy * dy > 25.0f)
         {
@@ -769,7 +789,7 @@ public:
         }
         else
         {
-            newPos = (mousePos - sliderRegionStart) / (double) sliderRegionSize;
+            newPos = (mousePos - (float) sliderRegionStart) / (double) sliderRegionSize;
 
             if (isVertical())
                 newPos = 1.0 - newPos;
@@ -794,7 +814,7 @@ public:
         auto maxSpeed = jmax (200.0, (double) sliderRegionSize);
         auto speed = jlimit (0.0, maxSpeed, (double) std::abs (mouseDiff));
 
-        if (speed != 0.0)
+        if (! approximatelyEqual (speed, 0.0))
         {
             speed = 0.2 * velocityModeSensitivity
                       * (1.0 + std::sin (MathConstants<double>::pi * (1.5 + jmin (0.5, velocityModeOffset
@@ -865,7 +885,7 @@ public:
                         popupDisplay->stopTimer();
                 }
 
-                currentDrag.reset (new DragInProgress (*this));
+                currentDrag = std::make_unique<ScopedDragNotification> (owner);
                 mouseDrag (e);
             }
         }
@@ -947,7 +967,7 @@ public:
         {
             restoreMouseIfHidden();
 
-            if (sendChangeOnlyOnRelease && valueOnMouseDown != static_cast<double> (currentValue.getValue()))
+            if (sendChangeOnlyOnRelease && ! approximatelyEqual (valueOnMouseDown, static_cast<double> (currentValue.getValue())))
                 triggerChangeMessage (sendNotificationAsync);
 
             currentDrag.reset();
@@ -994,6 +1014,38 @@ public:
         popupDisplay.reset();
     }
 
+    bool keyPressed (const KeyPress& key)
+    {
+        if (key.getModifiers().isAnyModifierKeyDown())
+            return false;
+
+        const auto getInterval = [this]
+        {
+            if (auto* accessibility = owner.getAccessibilityHandler())
+                if (auto* valueInterface = accessibility->getValueInterface())
+                    return valueInterface->getRange().getInterval();
+
+            return getStepSize (owner);
+        };
+
+        const auto valueChange = [&]
+        {
+            if (key == KeyPress::rightKey || key == KeyPress::upKey)
+                return getInterval();
+
+            if (key == KeyPress::leftKey || key == KeyPress::downKey)
+                return -getInterval();
+
+            return 0.0;
+        }();
+
+        if (approximatelyEqual (valueChange, 0.0))
+            return false;
+
+        setValue (getValue() + valueChange, sendNotificationSync);
+        return true;
+    }
+
     void showPopupDisplay()
     {
         if (style == IncDecButtons)
@@ -1010,25 +1062,36 @@ public:
                                             | ComponentPeer::windowIgnoresKeyPresses
                                             | ComponentPeer::windowIgnoresMouseClicks);
 
-            if (style == SliderStyle::TwoValueHorizontal
-                || style == SliderStyle::TwoValueVertical)
-            {
-                updatePopupDisplay (sliderBeingDragged == 2 ? getMaxValue()
-                                                            : getMinValue());
-            }
-            else
-            {
-                updatePopupDisplay (getValue());
-            }
-
+            updatePopupDisplay();
             popupDisplay->setVisible (true);
         }
     }
 
-    void updatePopupDisplay (double valueToShow)
+    void updatePopupDisplay()
     {
-        if (popupDisplay != nullptr)
-            popupDisplay->updatePosition (owner.getTextFromValue (valueToShow));
+        if (popupDisplay == nullptr)
+            return;
+
+        const auto valueToShow = [this]
+        {
+            constexpr SliderStyle multiSliderStyles[] { SliderStyle::TwoValueHorizontal,
+                                                        SliderStyle::TwoValueVertical,
+                                                        SliderStyle::ThreeValueHorizontal,
+                                                        SliderStyle::ThreeValueVertical };
+
+            if (std::find (std::begin (multiSliderStyles), std::end (multiSliderStyles), style) == std::end (multiSliderStyles))
+                return getValue();
+
+            if (sliderBeingDragged == 2)
+                return getMaxValue();
+
+            if (sliderBeingDragged == 1)
+                return getMinValue();
+
+            return getValue();
+        }();
+
+        popupDisplay->updatePosition (owner.getTextFromValue (valueToShow));
     }
 
     bool canDoubleClickToValue() const
@@ -1043,7 +1106,7 @@ public:
     {
         if (canDoubleClickToValue())
         {
-            DragInProgress drag (*this);
+            ScopedDragNotification drag (owner);
             setValue (doubleClickReturnValue, sendNotificationSync);
         }
     }
@@ -1082,11 +1145,11 @@ public:
                     auto delta = getMouseWheelDelta (value, (std::abs (wheel.deltaX) > std::abs (wheel.deltaY)
                                                                   ? -wheel.deltaX : wheel.deltaY)
                                                                * (wheel.isReversed ? -1.0f : 1.0f));
-                    if (delta != 0.0)
+                    if (! approximatelyEqual (delta, 0.0))
                     {
                         auto newValue = value + jmax (normRange.interval, std::abs (delta)) * (delta < 0 ? -1.0 : 1.0);
 
-                        DragInProgress drag (*this);
+                        ScopedDragNotification drag (owner);
                         setValue (owner.snapValue (newValue, notDragging), sendNotificationSync);
                     }
                 }
@@ -1141,8 +1204,8 @@ public:
                 {
                     auto pixelPos = (float) getLinearSliderPos (pos);
 
-                    mousePos = owner.localPointToGlobal (Point<float> (isHorizontal() ? pixelPos : (owner.getWidth() / 2.0f),
-                                                                       isVertical()   ? pixelPos : (owner.getHeight() / 2.0f)));
+                    mousePos = owner.localPointToGlobal (Point<float> (isHorizontal() ? pixelPos : ((float) owner.getWidth()  / 2.0f),
+                                                                       isVertical()   ? pixelPos : ((float) owner.getHeight() / 2.0f)));
                 }
 
                 const_cast <MouseInputSource&> (ms).setScreenPosition (mousePos);
@@ -1176,12 +1239,6 @@ public:
                                      getLinearSliderPos (lastValueMax),
                                      style, owner);
             }
-
-            if ((style == LinearBar || style == LinearBarVertical) && valueBox == nullptr)
-            {
-                g.setColour (owner.findColour (Slider::textBoxOutlineColourId));
-                g.drawRect (0, 0, owner.getWidth(), owner.getHeight(), 1);
-            }
         }
     }
 
@@ -1211,7 +1268,6 @@ public:
     }
 
     //==============================================================================
-
     void resizeIncDecButtons()
     {
         auto buttonRect = sliderRect;
@@ -1258,11 +1314,12 @@ public:
     int pixelsForFullDragExtent = 250;
     Time lastMouseWheelTime;
     Rectangle<int> sliderRect;
-    std::unique_ptr<DragInProgress> currentDrag;
+    std::unique_ptr<ScopedDragNotification> currentDrag;
 
     TextEntryBoxPosition textBoxPos;
     String textSuffix;
     int numDecimalPlaces = 7;
+    int fixedNumDecimalPlaces = -1;
     int textBoxWidth = 80, textBoxHeight = 20;
     IncDecButtonMode incDecButtonMode = incDecButtonsNotDraggable;
     ModifierKeys::Flags modifierToSwapModes = ModifierKeys::ctrlAltCommandModifiers;
@@ -1290,8 +1347,8 @@ public:
     std::unique_ptr<Button> incButton, decButton;
 
     //==============================================================================
-    struct PopupDisplayComponent  : public BubbleComponent,
-                                    public Timer
+    struct PopupDisplayComponent final : public BubbleComponent,
+                                         public Timer
     {
         PopupDisplayComponent (Slider& s, bool isOnDesktop)
             : owner (s),
@@ -1358,6 +1415,18 @@ public:
     }
 };
 
+//==============================================================================
+Slider::ScopedDragNotification::ScopedDragNotification (Slider& s)
+    : sliderBeingDragged (s)
+{
+    sliderBeingDragged.pimpl->sendDragStart();
+}
+
+Slider::ScopedDragNotification::~ScopedDragNotification()
+{
+    if (sliderBeingDragged.pimpl != nullptr)
+        sliderBeingDragged.pimpl->sendDragEnd();
+}
 
 //==============================================================================
 Slider::Slider()
@@ -1500,10 +1569,11 @@ void Slider::lookAndFeelChanged()   { pimpl->lookAndFeelChanged (getLookAndFeel(
 void Slider::enablementChanged()    { repaint(); pimpl->updateTextBoxEnablement(); }
 
 //==============================================================================
-Range<double> Slider::getRange() const noexcept  { return { pimpl->normRange.start, pimpl->normRange.end }; }
-double Slider::getMaximum() const noexcept       { return pimpl->normRange.end; }
-double Slider::getMinimum() const noexcept       { return pimpl->normRange.start; }
-double Slider::getInterval() const noexcept      { return pimpl->normRange.interval; }
+NormalisableRange<double> Slider::getNormalisableRange() const noexcept { return pimpl->normRange; }
+Range<double> Slider::getRange() const noexcept                         { return { pimpl->normRange.start, pimpl->normRange.end }; }
+double Slider::getMaximum() const noexcept                              { return pimpl->normRange.end; }
+double Slider::getMinimum() const noexcept                              { return pimpl->normRange.start; }
+double Slider::getInterval() const noexcept                             { return pimpl->normRange.interval; }
 
 void Slider::setRange (double newMin, double newMax, double newInt)      { pimpl->setRange (newMin, newMax, newInt); }
 void Slider::setRange (Range<double> newRange, double newInt)            { pimpl->setRange (newRange.getStart(), newRange.getEnd(), newInt); }
@@ -1610,11 +1680,14 @@ double Slider::snapValue (double attemptedValue, DragMode)
     return attemptedValue;
 }
 
-int Slider::getNumDecimalPlacesToDisplay() const noexcept   { return pimpl->numDecimalPlaces; }
+int Slider::getNumDecimalPlacesToDisplay() const noexcept
+{
+    return pimpl->getNumDecimalPlacesToDisplay();
+}
 
 void Slider::setNumDecimalPlacesToDisplay (int decimalPlacesToDisplay)
 {
-    pimpl->numDecimalPlaces = decimalPlacesToDisplay;
+    pimpl->setNumDecimalPlacesToDisplay (decimalPlacesToDisplay);
     updateText();
 }
 
@@ -1628,6 +1701,7 @@ void Slider::valueChanged() {}
 void Slider::setPopupMenuEnabled (bool menuEnabled)         { pimpl->menuEnabled = menuEnabled; }
 void Slider::setScrollWheelEnabled (bool enabled)           { pimpl->scrollWheelEnabled = enabled; }
 
+bool Slider::isScrollWheelEnabled() const noexcept          { return pimpl->scrollWheelEnabled; }
 bool Slider::isHorizontal() const noexcept                  { return pimpl->isHorizontal(); }
 bool Slider::isVertical() const noexcept                    { return pimpl->isVertical(); }
 bool Slider::isRotary() const noexcept                      { return pimpl->isRotary(); }
@@ -1652,6 +1726,9 @@ void Slider::mouseExit (const MouseEvent&)      { pimpl->mouseExit(); }
 // it is shown when dragging the mouse over a slider and releasing
 void Slider::mouseEnter (const MouseEvent&)     { pimpl->mouseMove(); }
 
+/** @internal */
+bool Slider::keyPressed (const KeyPress& k)     { return pimpl->keyPressed (k); }
+
 void Slider::modifierKeysChanged (const ModifierKeys& modifiers)
 {
     if (isEnabled())
@@ -1676,4 +1753,75 @@ void Slider::mouseWheelMove (const MouseEvent& e, const MouseWheelDetails& wheel
         Component::mouseWheelMove (e, wheel);
 }
 
+//==============================================================================
+class SliderAccessibilityHandler final : public AccessibilityHandler
+{
+public:
+    explicit SliderAccessibilityHandler (Slider& sliderToWrap)
+        : AccessibilityHandler (sliderToWrap,
+                                AccessibilityRole::slider,
+                                AccessibilityActions{},
+                                AccessibilityHandler::Interfaces { std::make_unique<ValueInterface> (sliderToWrap) }),
+          slider (sliderToWrap)
+    {
+    }
+
+    String getHelp() const override   { return slider.getTooltip(); }
+
+private:
+    class ValueInterface final : public AccessibilityValueInterface
+    {
+    public:
+        explicit ValueInterface (Slider& sliderToWrap)
+            : slider (sliderToWrap),
+              useMaxValue (slider.isTwoValue())
+        {
+        }
+
+        bool isReadOnly() const override  { return false; }
+
+        double getCurrentValue() const override
+        {
+            return useMaxValue ? slider.getMaximum()
+                               : slider.getValue();
+        }
+
+        void setValue (double newValue) override
+        {
+            Slider::ScopedDragNotification drag (slider);
+
+            if (useMaxValue)
+                slider.setMaxValue (newValue, sendNotificationSync);
+            else
+                slider.setValue (newValue, sendNotificationSync);
+        }
+
+        String getCurrentValueAsString() const override          { return slider.getTextFromValue (getCurrentValue()); }
+        void setValueAsString (const String& newValue) override  { setValue (slider.getValueFromText (newValue)); }
+
+        AccessibleValueRange getRange() const override
+        {
+            return { { slider.getMinimum(), slider.getMaximum() },
+                     getStepSize (slider) };
+        }
+
+    private:
+        Slider& slider;
+        const bool useMaxValue;
+
+        //==============================================================================
+        JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (ValueInterface)
+    };
+
+    Slider& slider;
+
+    //==============================================================================
+    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (SliderAccessibilityHandler)
+};
+
+std::unique_ptr<AccessibilityHandler> Slider::createAccessibilityHandler()
+{
+    return std::make_unique<SliderAccessibilityHandler> (*this);
+}
+
 } // namespace juce
