@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-7-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -30,7 +29,7 @@ namespace juce
 const char* const Toolbar::toolbarDragDescriptor = "_toolbarItem_";
 
 //==============================================================================
-class Toolbar::Spacer  : public ToolbarItemComponent
+class Toolbar::Spacer final : public ToolbarItemComponent
 {
 public:
     Spacer (int itemID, float sizeToUse, bool shouldDrawBar)
@@ -52,7 +51,7 @@ public:
         }
         else
         {
-            maxSize = roundToInt (toolbarThickness * fixedSize);
+            maxSize = roundToInt ((float) toolbarThickness * fixedSize);
             minSize = drawBar ? maxSize : jmin (4, maxSize);
             preferredSize = maxSize;
 
@@ -88,9 +87,9 @@ public:
             auto thickness = 0.2f;
 
             if (isToolbarVertical())
-                g.fillRect (w * 0.1f, h * (0.5f - thickness * 0.5f), w * 0.8f, h * thickness);
+                g.fillRect ((float) w * 0.1f, (float) h * (0.5f - thickness * 0.5f), (float) w * 0.8f, (float) h * thickness);
             else
-                g.fillRect (w * (0.5f - thickness * 0.5f), h * 0.1f, w * thickness, h * 0.8f);
+                g.fillRect ((float) w * (0.5f - thickness * 0.5f), (float) h * 0.1f, (float) w * thickness, (float) h * 0.8f);
         }
 
         if (getEditingMode() != normalMode && ! drawBar)
@@ -107,33 +106,33 @@ public:
 
                 if (isToolbarVertical())
                 {
-                    x1 = w * 0.5f;
-                    y1 = h * 0.4f;
+                    x1 = (float) w * 0.5f;
+                    y1 = (float) h * 0.4f;
                     x2 = x1;
-                    y2 = indentX * 2.0f;
+                    y2 = (float) indentX * 2.0f;
 
                     x3 = x1;
-                    y3 = h * 0.6f;
+                    y3 = (float) h * 0.6f;
                     x4 = x1;
-                    y4 = h - y2;
+                    y4 = (float) h - y2;
 
-                    hw = w * 0.15f;
-                    hl = w * 0.2f;
+                    hw = (float) w * 0.15f;
+                    hl = (float) w * 0.2f;
                 }
                 else
                 {
-                    x1 = w * 0.4f;
-                    y1 = h * 0.5f;
-                    x2 = indentX * 2.0f;
+                    x1 = (float) w * 0.4f;
+                    y1 = (float) h * 0.5f;
+                    x2 = (float) indentX * 2.0f;
                     y2 = y1;
 
-                    x3 = w * 0.6f;
+                    x3 = (float) w * 0.6f;
                     y3 = y1;
-                    x4 = w - x2;
+                    x4 = (float) w - x2;
                     y4 = y1;
 
-                    hw = h * 0.15f;
-                    hl = h * 0.2f;
+                    hw = (float) h * 0.15f;
+                    hl = (float) h * 0.2f;
                 }
 
                 Path p;
@@ -152,7 +151,7 @@ private:
 };
 
 //==============================================================================
-class Toolbar::MissingItemsComponent  : public PopupMenu::CustomComponent
+class Toolbar::MissingItemsComponent final : public PopupMenu::CustomComponent
 {
 public:
     MissingItemsComponent (Toolbar& bar, int h)
@@ -162,9 +161,9 @@ public:
     {
         for (int i = bar.items.size(); --i >= 0;)
         {
-            auto* tc = bar.items.getUnchecked(i);
+            auto* tc = bar.items.getUnchecked (i);
 
-            if (dynamic_cast<Spacer*> (tc) == nullptr && ! tc->isVisible())
+            if (tc != nullptr && dynamic_cast<Spacer*> (tc) == nullptr && ! tc->isVisible())
             {
                 oldIndexes.insert (0, i);
                 addAndMakeVisible (tc, 0);
@@ -243,11 +242,8 @@ private:
 //==============================================================================
 Toolbar::Toolbar()
 {
-    missingItemsButton.reset (getLookAndFeel().createToolbarMissingItemsButton (*this));
-    addChildComponent (missingItemsButton.get());
-
-    missingItemsButton->setAlwaysOnTop (true);
-    missingItemsButton->onClick = [this] { showMissingItems(); };
+    lookAndFeelChanged();
+    initMissingItemButton();
 }
 
 Toolbar::~Toolbar()
@@ -389,7 +385,7 @@ String Toolbar::toString() const
     String s ("TB:");
 
     for (int i = 0; i < getNumItems(); ++i)
-        s << getItemId(i) << ' ';
+        s << getItemId (i) << ' ';
 
     return s.trimEnd();
 }
@@ -535,6 +531,16 @@ void Toolbar::updateAllItemPositions (bool animate)
 }
 
 //==============================================================================
+void Toolbar::initMissingItemButton()
+{
+    if (missingItemsButton == nullptr)
+        return;
+
+    addChildComponent (*missingItemsButton);
+    missingItemsButton->setAlwaysOnTop (true);
+    missingItemsButton->onClick = [this] { showMissingItems(); };
+}
+
 void Toolbar::showMissingItems()
 {
     jassert (missingItemsButton->isShowing());
@@ -543,7 +549,7 @@ void Toolbar::showMissingItems()
     {
         PopupMenu m;
         auto comp = std::make_unique<MissingItemsComponent> (*this, getThickness());
-        m.addCustomItem (1, std::move (comp));
+        m.addCustomItem (1, std::move (comp), nullptr, TRANS ("Additional Items"));
         m.showMenuAsync (PopupMenu::Options().withTargetComponent (missingItemsButton.get()));
     }
 }
@@ -641,14 +647,23 @@ void Toolbar::itemDropped (const SourceDetails& dragSourceDetails)
         tc->setState (Button::buttonNormal);
 }
 
+void Toolbar::lookAndFeelChanged()
+{
+    missingItemsButton.reset (getLookAndFeel().createToolbarMissingItemsButton (*this));
+    initMissingItemButton();
+}
+
 void Toolbar::mouseDown (const MouseEvent&) {}
 
 //==============================================================================
-class Toolbar::CustomisationDialog   : public DialogWindow
+class Toolbar::CustomisationDialog final : public DialogWindow
 {
 public:
     CustomisationDialog (ToolbarItemFactory& factory, Toolbar& bar, int optionFlags)
-        : DialogWindow (TRANS("Add/remove items from toolbar"), Colours::white, true, true),
+        : DialogWindow (TRANS ("Add/remove items from toolbar"),
+                        bar.findColour (Toolbar::customisationDialogBackgroundColourId),
+                        true,
+                        true),
           toolbar (bar)
     {
         setContentOwned (new CustomiserPanel (factory, toolbar, optionFlags), true);
@@ -670,7 +685,7 @@ public:
     bool canModalEventBeSentToComponent (const Component* comp) override
     {
         return toolbar.isParentOf (comp)
-                 || dynamic_cast<const ToolbarItemComponent::ItemDragAndDropOverlayComponent*> (comp) != nullptr;
+                 || dynamic_cast<const detail::ToolbarItemDragAndDropOverlayComponent*> (comp) != nullptr;
     }
 
     void positionNearBar()
@@ -721,16 +736,17 @@ private:
                 addAndMakeVisible (styleBox);
                 styleBox.setEditableText (false);
 
-                if ((optionFlags & Toolbar::allowIconsOnlyChoice) != 0)     styleBox.addItem (TRANS("Show icons only"), 1);
-                if ((optionFlags & Toolbar::allowIconsWithTextChoice) != 0) styleBox.addItem (TRANS("Show icons and descriptions"), 2);
-                if ((optionFlags & Toolbar::allowTextOnlyChoice) != 0)      styleBox.addItem (TRANS("Show descriptions only"), 3);
+                if ((optionFlags & Toolbar::allowIconsOnlyChoice) != 0)     styleBox.addItem (TRANS ("Show icons only"), 1);
+                if ((optionFlags & Toolbar::allowIconsWithTextChoice) != 0) styleBox.addItem (TRANS ("Show icons and descriptions"), 2);
+                if ((optionFlags & Toolbar::allowTextOnlyChoice) != 0)      styleBox.addItem (TRANS ("Show descriptions only"), 3);
 
                 int selectedStyle = 0;
                 switch (bar.getStyle())
                 {
-                    case Toolbar::iconsOnly:        selectedStyle = 1; break;
-                    case Toolbar::iconsWithText:    selectedStyle = 2; break;
-                    case Toolbar::textOnly:         selectedStyle = 3; break;
+                    case Toolbar::iconsOnly:      selectedStyle = 1; break;
+                    case Toolbar::iconsWithText:  selectedStyle = 2; break;
+                    case Toolbar::textOnly:       selectedStyle = 3; break;
+                    default:                      break;
                 }
 
                 styleBox.setSelectedId (selectedStyle);
@@ -757,6 +773,7 @@ private:
                 case 1:   toolbar.setStyle (Toolbar::iconsOnly); break;
                 case 2:   toolbar.setStyle (Toolbar::iconsWithText); break;
                 case 3:   toolbar.setStyle (Toolbar::textOnly); break;
+                default:  break;
             }
 
             palette.resized(); // to make it update the styles
@@ -803,4 +820,10 @@ void Toolbar::showCustomisationDialog (ToolbarItemFactory& factory, const int op
         ->enterModalState (true, nullptr, true);
 }
 
+//==============================================================================
+std::unique_ptr<AccessibilityHandler> Toolbar::createAccessibilityHandler()
+{
+    return std::make_unique<AccessibilityHandler> (*this, AccessibilityRole::group);
+}
+
 } // namespace juce
